/*
 * Panasync
 *
 * Copyright (C) 2000 Carlos Baquero, Victor Fonte,
 * Paulo Sergio Almeida, Universidade do Minho
 *
 * panasync@gsd.di.uminho.pt - http://gsd.di.uminho.pt/panasync
 *
 * Departamento de Informatica, Universidade do Minho
 * Campus de Gualtar, 4710-057 Braga, Portugal
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 *
 */

#include <stl.h>
#include <string>
#include <panadebug.h>

////	unix: id = `find /proc/ -type f -exec cat {} \; 2> /dev/null | md5sum`

class	pts
{
	public:

	pts(string kind = "vff", string id = "cbm", string counter = "psa");
	pts(const pts & source);

	~pts();

	void	set_dirty();
	pts	fork();
	pts &	join(const pts & relative);
	bool	dominates(const pts & relative) const;
	bool	equivalent(const pts & relative) const;
	bool	uncomparable(const pts & relative) const;

	template <class Stream> void composite(Stream& stream);

	private:

	//	definition of fragment types
	//
	struct base {
		string	kind;
		string	id;
		string	counter;
	};

	typedef vector<bool> part;
	typedef set<part> frag;

	//	declaration of the fragment objects
	//
	base	bs;
	frag	ff;
	frag	sf;

	//	private methods
	//
	void	distribute(frag & ds, frag & fs1, frag & fs2, frag & fs3);

	//	friends
	//
	friend ostream& operator<< (ostream& out, const pts& pts);

	//	debug
	//
#ifdef	__panasync_debug_causal_history
	set<int> ch;
#endif	__panasync_debug_causal_history
};

#include <panasync-code.h>
